package TWiki::Contrib::TWikiShellContrib::Ext::Sdk;

use TWiki::Contrib::TWikiShellContrib::Common;
use TWiki::Contrib::TWikiShellContrib::DirHandling;
use TWiki::Contrib::TWikiShellContrib::Zip;
use File::Copy;
use File::Path;
 
$defaultDevelop="develop";
$defaultBase="base";
$defaultIncoming="incoming";
$defaultDistro="distros";
$defaultExt="extensions";
$defaultCheckout="checkout";
$defaultPlugins="twikiplugins";

sub run_help {
    my $shell=shift;
	my $config=shift;
	my @params=@_;
	my $params=join(" ",@params);
	if ($params) {
		$shell->cmd("help sdk $params");
		return;
	}
    my %handlers=%{$shell->{handlers}};
    print "  Commands:\n";
    foreach my $handler (keys %handlers) {
        if ($handler =~ /^sdk /) {
            print "    ".$handler." - ";

            my $documented=0;
            my %sdkHandlers=%{$handlers{$handler}};
            foreach my $sdkHandler (keys %sdkHandlers) {
                if ($sdkHandler =~ /smry/ && defined &{$sdkHandlers{$sdkHandler}}) {
                    print &{$sdkHandlers{$sdkHandler}}."\n";
                    $documented=1;
                } 
            }
            
            if (!$documented) {
                print "not finished - undocumented\n";
            }
        }
    }
    print "\n";
}

sub smry_help {
    return " Display this help";
}

#--------------------------------------------------------------------------------------------------

sub run_download {
    my ($shell,$config,$type,$name) = @_;
    print "Hello World\n";
}

#--------------------------------------------------------------------------------------------------

sub run_test{
    my ($shell,$config,$type,$name) = @_;
    print "Hello World\n";
}

#--------------------------------------------------------------------------------------------------

sub run_unzip_distro {
    my ($shell,$config,@args)=@_;
    unzipPackage($shell,$config,"base","DISTRO",@args);
}

###################################################

sub unzipPackage {
    my ($shell,$config,$base,$type,$zip,$destDir)=@_;
    
    $destDir=$zip if !$destDir;
   
    my $incomingDir=$config->{SDK}{uc $type}{dir};
    if (!$incomingDir) { 
        $shell->printTerse("Unknown type $type\n");
        return;
    }

    my $sourceFile="$incomingDir/$zip.zip";
    
    $shell->printVerbose("Installing $type on directory $base\n");
    $base =uc $base;
    unzip($config,$sourceFile,$config->{SDK}{$base}{dir}."/$destDir");    
}

###################################################################################################

sub onImport {
    my ($shell) = @_;
    $shell->importCommand("Sdk::Install");
    $shell->importCommand("Sdk::Uninstall");
    $shell->importCommand("Sdk::Develop");
    _checkIncomingDir(@_);    
    _checkDestinationDir(@_);
    _checkApacheDir(@_);
    
}

#--------------------------------------------------------------------------------------------------

sub _checkApacheDir {
    my ($shell,$config)=@_;
    my $apacheDefault="/etc/apache/httpd.conf";
 
    $config->{SDK}{BASE}{Apache}=askUser($config->{SDK}{BASE}{Apache},
                                $apacheDefault,
                                "Please tell me the complete absolute path to the httpd.conf file for the base installation",
                                sub {return (-e shift)});
                                       
    $config->{SDK}{DEVELOP}{Apache}=askUser($config->{SDK}{DEVELOP}{Apache},
                               $apacheDefault,
                               "Please tell me the complete absolute path to the httpd.conf file for the development installation",
                               sub {return (-e shift)});
}

#--------------------------------------------------------------------------------------------------

sub _checkIncomingDir {
    my ($shell,$config)=@_;
    
    $config->{SDK}{EXT}{dir} = askUser($config->{SDK}{EXT}{dir},
                                   "/home/twiki/incoming/extensions",
                                   "Please tell me the absolute path where the extensions zip files are stored",
                                   sub {my $file=shift;print "-->".$file."\n";print "-->".(-d $file)."\n";return (-d $file)});
                                      
    $config->{SDK}{DISTRO}{dir} = askUser($config->{SDK}{DISTRO}{dir},
                                   "/home/twiki/incoming/distros",
                                   "Please tell me the absolute path where the extensions zip files are stored",
                                   sub {return (-d shift)});
    
    $config->save();
   
}

#--------------------------------------------------------------------------------------------------

sub _checkDestinationDir {
    my ($shell,$config)=@_;
    
    $config->{SDK}{BASE}{dir} = askUser($config->{SDK}{BASE}{dir},
                               "/home/twiki/$defaultBase",
                               "Please tell me the absolute path where the base installations are stored",
                               sub {return (-d shift)});

    $config->{SDK}{DEVELOP}{dir} = askUser($config->{SDK}{DEVELOP}{dir},
                               "/home/twiki/$defaultDevelop",
                               "Please tell me the absolute path where the development installations are stored",
                               sub {return (-d shift)});
    
    $config->save();
}

###################################################################################################

1;
