package TWiki::Contrib::TWikiShellContrib::Ext::Sdk::Develop;

use TWiki::Contrib::TWikiShellContrib::Common;
use TWiki::Contrib::TWikiShellContrib::DirHandling;
use TWiki::Contrib::TWikiShellContrib::Zip;
use TWiki::Contrib::TWikiShellContrib::Ext::Sdk;
use File::Copy;

sub run {
    my ($shell,$config,$distroName,$targetName) = @_;
    $targetName=$distroName if !$targetName;
    #TODO: Check if already exists
    TWiki::Contrib::TWikiShellContrib::Ext::Sdk::unzipPackage($shell,$config,"DEVELOP","DISTRO",$distroName,$targetName);
    my ($server,$port)=_changeApacheConfig($shell,$config,$targetName,"DEVELOP");
    _changeTwikiConfig($shell,$config,$targetName,"DEVELOP",$server.":".$port);
    
}

sub smry {
	return "Installs a distro in the develop directory.";
}

sub help {
	return smry()." 
	Usage: sdk develop <distroname> <targetname>;
	 Will install a distro from <distroname>.zip 
                      to \${developDir}/<targetname>
                      
            .-  The apache http.conf will be modified to point to this
            installation.
            .-  The distro TWiki.cfg will be modified accordingly.
";
}
	
###################################################

sub _changeApacheConfig {
    my ($shell,$config,$name, $base)=@_;

    my $apacheCfgFile=$config->{SDK}{$base}{Apache};
    my ($server,$port)=("localhost","80");
        
    $shell->printVerbose("Modifying $apacheCfgFile\n");
    if (! -e $apacheCfgFile) {
        print "$apacheCfgFile not found\n";
        return;
    }

    $shell->printVerbose("   * Creating  backup of $apacheCfgFile\n");
    my $backupFile=$apacheCfgFile.".sdk".time."~";
    copy($apacheCfgFile,$backupFile);
    $shell->printVerbose("   * Writing new $apacheCfgFile\n");
    open INBASE,"<$backupFile";
    my @lines=<INBASE>;
    close INBASE;

    open OUTBASE,">$apacheCfgFile";
    my $modAlias=0;
    my $basePath=lc $base."/$name";
    my $firstLine="##################################BEGIN $basePath";
    my $quickFinish=0;
    foreach my $line (@lines) {
        if (!$quickFinish) {
            if ($line =~ /Port\s+([0-9]+)/) {
                $port=$1;
            } elsif ($line =~ /ServerName \s+(.+)$/) {
                $server=$1;
            } elsif ($line =~ /<IfModule mod_alias.c>/) {
                $modAlias=1;
            } elsif ($line =~ /$firstLine/) {
                $shell->printVerbose("   * $basePath already configured in $apacheCfgFile. No actions taken\n");
                $quickFinish=1;
                $modAlias=2;
            } elsif (($line =~ /\<\/IfModule\>/) && $modAlias eq 1) {
                $modAlias=2;
                print OUTBASE "
    $firstLine
    alias /$basePath /home/twiki/$basePath

    <Directory \"/home/twiki/$basePath\">
         AllowOverride All
         Allow From All
    </Directory>

    alias /$basePath/bin /home/twiki/$basePath/bin
    
    <Directory \"/home/twiki/$basePath/bin\">
         AllowOverride All
         Allow From All
        Options ExecCGI
        SetHandler cgi-script
    </Directory>
    #################################################
";
            }
        }        
        print OUTBASE $line;
    }
    close OUTBASE;
    if ($modAlias!=2) {
        $shell->printVerbose("   * There is no mod_alias.c section in $apacheCfgFile. Please, create it manually and rerun the process\n");            
    }
    
    return ($server,$port);
}

#--------------------------------------------------------------------------------------------------

sub _changeTwikiConfig {
    my ($shell,$config,$name, $base,$serverUrl)=@_;
    
    my $twikiBaseDir=$config->{SDK}{$base}{dir}."/$name";
    my $twikiBaseUrl="/".lc $base."/$name";
    my $twikiCfgFile="$twikiBaseDir/lib/TWiki.cfg";
    
    $shell->printVerbose("Modifying  $twikiCfgFile\n");
    if (! -e $twikiCfgFile) {
        print "$twikiCfgFile not found\n";
        return;
    }

    $shell->printVerbose("   * Creating  backup of $twikiCfgFile\n");
    my $backupFile=$twikiCfgFile.".sdk".time."~";
    copy($twikiCfgFile,$backupFile);
    $shell->printVerbose("   * Writing new $twikiCfgFile\n");
    open INBASE,"<$backupFile";
    my @lines=<INBASE>;
    close INBASE;

    open OUTBASE,">$twikiCfgFile";
    foreach my $line (@lines) {
        if ($line=~ /(.*?)(\s+)\=(\s+)(.*)/) {
            if ($1 eq '$scriptUrlPath') {
                $line = "$1$2=$3\"$twikiBaseUrl/bin\";\n";
            } elsif ($1 eq '$pubUrlPath') {
                $line = "$1$2=$3\"$twikiBaseUrl/pub\";\n";
            } elsif ($1 eq '$pubDir') {
                $line = "$1$2=$3\"$twikiBaseDir/pub\";\n";
            } elsif ($1 eq '$templateDir') {
                $line = "$1$2=$3\"$twikiBaseDir/templates\";\n";
            } elsif ($1 eq '$dataDir') {
                $line = "$1$2=$3\"$twikiBaseDir/data\";\n";
            } elsif ($1 eq '$wikiHomeUrl') {
                $line = "$1$2=$3\"http://$serverUrl$twikiBaseUrl/bin/view\";\n";
            } elsif ($1 eq '$defaultUrlHost') {
                $line = "$1$2=$3\"http://$serverUrl\";\n";
            } 
        }
        print OUTBASE $line;
    }
    
    close OUTBASE;
}


###################################################################################################


1;
